from rich import print
import sys
import cv2
import time
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL
from util.track import *
from events.events_engine import EventsEngine
from enum import Enum

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class ProductivityMonitoring(Scenario):
    alert_time = 10
    def __init__(self, scenario_name, camera_name=0, events=None, triton_url=TRITON_HTTP_URL):
        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton(triton_url, scenario_name)
        self.f_event =  EventsEngine(use_redis=False)
        self.line_thickness=3
        super().__init__(scenario_name, camera_name, events, triton_url)

    def draw_boxes(self, img, bbox, count, total_time, offset=(0, 0)):
        for i, box in enumerate(bbox):
            x1, y1, x2, y2 = [int(i) for i in box]
            x1 += offset[0]
            x2 += offset[0]
            y1 += offset[1]
            y2 += offset[1]
            data = (int((box[0]+box[2])/2),(int((box[1]+box[3])/2)))
            label = f"ID: {str(count)} - Duration: {str(int(total_time))}"
            (w, h), _ = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, 0.6, 1)
            cv2.rectangle(img, (x1, y1), (x2, y2),(255,191,0), 2)
            cv2.rectangle(img, (x1, y1 - 20), (x1 + w, y1), (255,191,0), -1)
            cv2.putText(img, label, (x1, y1 - 5),cv2.FONT_HERSHEY_SIMPLEX, 0.6,
            [255, 255, 255], 1)
            cv2.circle(img, data, 3, (255,191,0),-1)
        return img

    def start(self, camera_name=0):
        '''
        Stream processing
        When running a scenario - the caller can specify any specific camera.
        '''
        # pickup_region = [(0,0),(600,0),(600,1080),(0,1080)]
        # drop_region = [(1250,0),(1980,0),(1980,1080),(1250,1080)]
        table_region = [(300,0),(340,0),(340,480),(300,480)] # webcam
        stream = camera_name
        object_class = 39 # 0=Person, 39=bottle, 69=cell phone
        event_fire_threshold = 40
        text_font = 0.5
        text_thickness = 1
        shipping_threshold = 30
        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)
        info_dic = {}
        objects = set()
        sort_max_age = 5
        sort_min_hits = 2
        sort_iou_thresh = 0.2
        sort_tracker = Track(max_age=sort_max_age,
                        min_hits=sort_min_hits,
                        iou_threshold=sort_iou_thresh)
        prev_frame_time = time.time()
        new_frame_time = 0
        last_obj_time = time.time()
        curr_obj_time = 0
        object_ids = set()
        shipping_ev = False
        while True:
            # Do processing
            ret, frame = video.read()
            if 'rtsp' in str(stream):
                frame_width = 640
                frame_height = 480
                frame = cv2.resize(frame, (frame_width, frame_height))
            # draw region in image
            # frame = cv2.polylines(frame, [np.array(table_region).reshape((-1, 1, 2))], True, 0.5, 1)
            # fps = video.get(cv2.CAP_PROP_FPS)
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue
            cv2.putText(frame, "Box Count: " + str(len(object_ids)), (0,50) , cv2.FONT_HERSHEY_SIMPLEX, text_font, (0,0,250), text_thickness, cv2.LINE_AA)
            cv2.putText(frame, 'Shipping : '+str(shipping_ev), (0,70) , cv2.FONT_HERSHEY_SIMPLEX, text_font, (0,0,250), text_thickness, cv2.LINE_AA)
            new_frame_time = time.time()
            fps = 1/(new_frame_time-prev_frame_time)
            # total_time += (new_frame_time-prev_frame_time)
            prev_frame_time = new_frame_time
            curr_obj_time = time.time()
            # Detect smoke & fire
            results = self.model(frame, size=640)  # batched inference
            det = results.xyxy[0]
            print("-----------------------------------")
            if len(det):
                dets_to_sort = np.empty((0,6))
                for x1,y1,x2,y2,conf,detclass in det.cpu().detach().numpy():
                    if int(detclass) == object_class: # 0=Person, 39=bottle, 69=cell phone
                        dets_to_sort = np.vstack((dets_to_sort,
                                                np.array([x1, y1, x2, y2,
                                                            conf, detclass])))
                tracked_dets = sort_tracker.update(dets_to_sort)
                if len(tracked_dets)>0:
                    bbox_xyxy = tracked_dets[:,:4][0].tolist()
                    identities = tracked_dets[:, 8]
                    for i in range(len(identities)):
                        id = identities[i]
                        bbox = bbox_xyxy
                        inside_region = cv2.pointPolygonTest(np.array(table_region), (int(bbox_xyxy[0] + (bbox_xyxy[2]-bbox_xyxy[0])/2), int(bbox_xyxy[1] + (bbox_xyxy[3]-bbox_xyxy[1])/2)), False)
                        if inside_region > 0:
                            shipping_ev = True
                            self.draw_boxes(frame, [bbox], str(id), 0)
                            object_ids.add(id)
                            last_obj_time = curr_obj_time
                        cv2.putText(frame, "Box Count: " + str(len(object_ids)), (0,50) , cv2.FONT_HERSHEY_SIMPLEX, text_font, (0,0,250), text_thickness, cv2.LINE_AA)
                        cv2.putText(frame, 'Shipping : '+str(shipping_ev), (0,30) , cv2.FONT_HERSHEY_SIMPLEX, text_font, (0,0,250), text_thickness, cv2.LINE_AA)
                cv2.imshow('Output', frame)
                key = cv2.waitKey(5)
                if key == 27:
                    # import sys
                    print('Exiting.')
                    sys.exit(0)
            if (curr_obj_time - last_obj_time) > event_fire_threshold:
                last_obj_time = curr_obj_time
                print("fire event")
                self.f_event.fire_event(Event.INFO, 'Production', "Production-Monitoring", 'Production-Info', {"Production Status : " : shipping_ev, "Object Count : " : len(object_ids)})
            if (curr_obj_time - last_obj_time) > shipping_threshold: # and shipping_ev == True:
                last_obj_time = curr_obj_time
                shipping_ev = False
            results.print()
            if self.stop_evt.is_set():
                break

def camera_stream():
    snf = ProductivityMonitoring(scenario_name = 'productivity-detection')
    snf.start(camera_name=0)

if __name__ == '__main__':
    camera_stream()


import pytest
from typer.testing import CliRunner
from unittest.mock import patch
from rich import print
from cli import pipeline_app

from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd  
             
@pytest.fixture
def runner():
    return CliRunner()

def test_pipeline_create(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['create', '--name', 'test_pipe'])

        assert result.exit_code == 0
        assert 'Creating pipeline test_pipe' in mock_print.call_args_list[0][0][0]


def test_pipeline_add_scenario(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['add-scenario', '--pipeline', 'test_pipe', '--scenario', 'smoke-and-fire'])

        assert result.exit_code == 0
        assert 'Adding scenario smoke-and-fire to pipeline test_pipe' in mock_print.call_args_list[0][0][0]


def test_pipeline_add_preprocess(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['add-preprocess', '--pipeline', 'test_pipe', '--preprocess', 'face-blur'])

        assert result.exit_code == 0
        assert 'Adding preprocess face-blur to pipeline test_pipe' in mock_print.call_args_list[0][0][0]


def test_pipeline_add_camera(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['add-camera', '--pipeline', 'test_pipe', '--camera', 'OFFICE-01'])

        assert result.exit_code == 0
        assert 'Adding camera OFFICE-01 to pipeline test_pipe' in mock_print.call_args_list[0][0][0]


def test_pipeline_remove_camera(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['remove-camera', '--pipeline', 'test_pipe', '--camera', 'OFFICE-01'])

        assert result.exit_code == 0
        assert 'Removing camera OFFICE-01 to pipeline test_pipe' in mock_print.call_args_list[0][0][0]


def test_pipeline_reset(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['reset', '--pipeline', 'test_pipe'])

        assert result.exit_code == 0
        assert 'Pipeline test_pipe reset' in mock_print.call_args_list[0][0][0]


def test_pipeline_show(runner):
    with patch('main.print') as mock_print:
        result = runner.invoke(pipeline_app, ['show', '--pipeline', 'test_pipe'])

        assert result.exit_code == 0
if __name__=='__main__':
    CliRunner()


import unittest
from unittest.mock import patch
from io import StringIO
from cli.device_app import device_list, device_select, device_modules, device_gpu_mem_stats


from config import VISIONAI_EXEC
from util.general import WorkingDirectory, invoke_cmd

class TestDeviceApp(unittest.TestCase):
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    def test_device_list(self):
        expected_output = "Devices : ['oos-training', 'edge-dev1', 'edge-dev2']\n"
        with patch('sys.stdout', new=StringIO()) as mock_output:
            device_list()
            self.assertEqual(mock_output.getvalue(), expected_output)

    def test_device_select(self):
        expected_output = "Selecting device : edge-dev1\n"
        with patch('sys.stdout', new=StringIO()) as mock_output:
            device_select("edge-dev1")
            self.assertEqual(mock_output.getvalue(), expected_output)

    def test_device_modules(self):
        expected_output = "Listing modules : ['edgeCtl', 'edgeEvents']\n"
        with patch('sys.stdout', new=StringIO()) as mock_output:
            device_modules()
            self.assertEqual(mock_output.getvalue(), expected_output)

    def test_device_gpu_mem_stats(self):
        expected_output = "Getting gpu/mem stats\n"
        with patch('sys.stdout', new=StringIO()) as mock_output:
            device_gpu_mem_stats()
            self.assertEqual(mock_output.getvalue(), expected_output)

if __name__ == '__main__':
    unittest.main()



import unittest
import json, requests
import sys

from pathlib import Path
from config import SCENARIOS_OVERRIDE



class TestScenarioJson(unittest.TestCase):

    def test_scenario_json_file_exist(self):
        my_file = Path(SCENARIOS_OVERRIDE)
        assert my_file.is_file()

    def test_scenario_json_file_not_empty(self):
        my_file = Path(SCENARIOS_OVERRIDE)
        assert my_file.stat().st_size != 0


    def test_scenario_json_file_has_correct_format(self):
        with open(SCENARIOS_OVERRIDE,'r') as f:
            scenarios = json.load(f)
        assert isinstance(scenarios, dict)
        for scenario in scenarios['scenarios']:
            assert isinstance(scenario, dict)
            assert 'id' in scenario
            assert len(scenario['id']) > 0

            assert 'name' in scenario
            assert len(scenario['name']) > 0

            assert 'version' in scenario
            assert len(scenario['version']) > 0
            assert 'overview' in scenario
            assert 'docs' in scenario
            assert 'image' in scenario
            assert 'thumbnail' in scenario
            assert 'models' in scenario

            assert requests.get(scenario['image'], stream=True).status_code != 404
            assert requests.get(scenario['thumbnail'], stream=True).status_code != 404

            if(bool(scenario['models']['latest'])):
                assert requests.get(scenario['models']['latest']['model_url'], stream=True).status_code != 404

            assert isinstance(scenario['models'], dict)
            assert isinstance(scenario['models']['latest'], dict)
            assert isinstance(scenario['models']['other'], list)

            assert 'version' in scenario['models']['latest']
            assert len(scenario['models']['latest']['version']) > 0

            assert 'name' in scenario['models']['latest']
            assert len(scenario['models']['latest']['name']) > 0

            assert 'accuracy' in scenario['models']['latest']
            assert 'recall' in scenario['models']['latest']
            assert 'f1' in scenario['models']['latest']
            assert 'datasetSize' in scenario['models']['latest']

            assert 'model_url' in scenario['models']['latest']
            assert len(scenario['models']['latest']['model_url']) > 0

            assert isinstance(scenario['tags'], list)
            assert len(scenario['tags']) > 0

            assert isinstance(scenario['categories'], list)
            assert len(scenario['categories']) > 0


            assert isinstance(scenario['events'], list)
            assert len(scenario['events']) > 0

            assert isinstance(scenario['configuration'], list)


if __name__ == '__main__':
    unittest.main()






import json
import requests
import tempfile
import http.server
import socketserver
from pathlib import Path

FILE = Path(__file__).resolve()
CONFIGDIR = FILE.parents[0]  # config directory

# Define a mock URL and file for testing
SCENARIOS_OVERRIDE = tempfile.NamedTemporaryFile(delete=False).name
SCENARIOS_URL = "http://localhost:8000/scenarios.json"

def main():
    res = requests.get(SCENARIOS_URL)
    scenarios = res.json()
    with open(SCENARIOS_OVERRIDE, 'w') as f:
        json.dump(scenarios, f, indent=4)

    print(f'Scenarios schema {SCENARIOS_URL} downloaded to {SCENARIOS_OVERRIDE}')

def test_main():
    # Write some JSON data to a temporary file
    mock_scenarios = {"scenarios": [{"name": "scenario1"}, {"name": "scenario2"}]}
    mock_scenarios_file = tempfile.NamedTemporaryFile(delete=False)
    mock_scenarios_file.write(json.dumps(mock_scenarios).encode())
    mock_scenarios_file.close()

    # Start a local server to serve the temporary file
    Handler = http.server.SimpleHTTPRequestHandler
    with socketserver.TCPServer(("", 8000), Handler) as httpd:
        print("Serving mock scenarios at http://localhost:8000/scenarios.json")
        httpd.RequestHandlerClass.protocol_version = "HTTP/1.0"
        httpd.serve_forever()

    # Modify the URL to point to the local server
    global SCENARIOS_URL
    SCENARIOS_URL = "http://localhost:8000/scenarios.json"

    # Invoke the main function
    main()

    # Read the contents of the output file and compare to expected JSON data
    with open(SCENARIOS_OVERRIDE) as f:
        result = json.load(f)

    assert result == mock_scenarios



import unittest
from unittest.mock import patch
from io import StringIO
from cli import auth_app


class TestAuthApp(unittest.TestCase):

    def test_auth_status(self):
        with patch('sys.stdout', new=StringIO()) as mock_output:
            auth_app(['status'])
            self.assertIn('Print whether logged in or not', mock_output.getvalue())

    def test_login(self):
        with patch('sys.stdout', new=StringIO()) as mock_output:
            auth_app(['login', '--token', 'test-token'])
            self.assertIn('Logging using the authorization token: test-token', mock_output.getvalue())

    def test_logout(self):
        with patch('sys.stdout', new=StringIO()) as mock_output:
            auth_app(['logout'])
            self.assertIn('Logging out of current session', mock_output.getvalue())

if __name__ == '__main__':
    unittest.main()



import typer
from rich import print
from rich.console import Console


from util.docker_utils import *

err_console = Console(stderr=True)
web_app = typer.Typer()

@web_app.command('start')
def web_app_start():
    try:
        from config import check_visionai_api,service_communication,VISIONAI_API_URL
        service_status = check_visionai_api()
        if service_status:
            status = service_communication(f'{VISIONAI_API_URL}/webserver/start')
            print(status)
    except Exception as e:
        print(e)


@web_app.command('stop')
def web_app_stop():
    try:
        from config import check_visionai_api,service_communication,VISIONAI_API_URL
        service_status = check_visionai_api()
        if service_status:
            status = service_communication(f'{VISIONAI_API_URL}/webserver/stop')
            print(status)
    except Exception as e:
        print(e)


import os
import sys
from pathlib import Path
import unittest

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from visionai.scenarios.restricted_zone import RestrictedZone

class TestRestrictedZoneScenarios(unittest.TestCase):
    def test_ppe(self):
        rz = RestrictedZone()
        output = rz.start(camera_name=0, test=True)
        assert output is "10_SECS_RAN"

if __name__ == '__main__':
    unittest.main()


import docker
import unittest
from unittest.mock import patch
import sys
from pathlib import Path
from cli import stop_cmd

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
PKGDIR = FILE.parents[2] # visionai dir
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util.general import WorkingDirectory, invoke_cmd
from config import VISIONAI_EXEC

class TestStopCmd(unittest.TestCase):
    
    @WorkingDirectory(PKGDIR)
    def setUp(self):
        # Initialize visionai package before running tests
        output = invoke_cmd(f'{VISIONAI_EXEC} init')

    def test_stop_cmd(self):
        with patch('builtins.print') as test_print:
            with patch.object(docker.Container, 'stop') as test_stop:
                # Test case 1: stop all containers successfully
                test_stop.return_value = None
                stop_cmd()
                test_print.assert_called_with('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
                test_print.assert_called_with('Stop web-app....')
                test_print.assert_called_with('Stop API service....')
                test_print.assert_called_with('Stop redis server....')
                test_print.assert_called_with('Stop grafana server....')
                test_print.assert_called_with('Stop Triton server....')
                test_print.assert_called_with('Done.')

                # Test case 2: Web-app container not found
                test_stop.side_effect = docker.errors.NotFound()
                stop_cmd()
                test_print.assert_called_with('Web-app not running')

                # Test case 3: Web-API container not found
                test_stop.side_effect = [None, docker.errors.NotFound()]
                stop_cmd()
                test_print.assert_called_with('Web-API not running')

                # Test case 4: Redis container not found
                test_stop.side_effect = [None, None, docker.errors.NotFound()]
                stop_cmd()
                test_print.assert_called_with('Redis not running')

                # Test case 5: Grafana container not found
                test_stop.side_effect = [None, None, None, docker.errors.NotFound()]
                stop_cmd()
                test_print.assert_called_with('Grafana not running')

                # Test case 6: Triton container not found
                test_stop.side_effect = [None, None, None, None, docker.errors.NotFound()]
                stop_cmd()
                test_print.assert_called_with('Triton not running')
                
if __name__ == '__main__':
    unittest.main()
